// type TProcessDetailPageState = ProcessDetailPageState;
type TProcessDetailPageHandlers = ProcessDetailPageHandlers;
type TProcessDetailPageHelpers = ProcessDetailPageHelpers;
type TProcessDetailPageInit = ProcessDetailPageInit;
type TProcessDetailPageNodes = ProcessDetailPageNodes;
type TProcessDetailPageState = ProcessDetailPageState;
type TWaitlistCommentDialog = WaitlistCommentDialog;
