interface TSharedParams {
  addAttributeUrl: string;
  markMatchTypeUrl: string;
  markMatchedUrl: string;
  markAllMatchedUrl: string;
  multimatch: string;
  isWaitlist: boolean;
  currentRole: string;
  currentUser: string;
  currentProcess: number;
}
