interface TMatchProcess {
  comment: string; // ''
  location: string; // 'United States'
  name: string; // 'Home furniture - Cabinets and non-wood, nonupholstered; at manufacturer'
  unit: string; // 'USD'
}
