// @ts-check

/** Api base */
export const readCommentsApiUrl = '/comments/read';
export const resolveThreadApiUrl = '/comments/resolve-thread';
export const createCommentApiUrl = '/comments/create-comment';
export const addNewThreadApiUrl = '/comments/create-thread';
