:root {
  --thread-comments-toolbar-items-size: 38px;
}

/* Basic styles */
.panels-layout.process-detail-layout > *,
.process-detail > * {
  transition: all var(--common-animation-time);
}

.process-detail {
  position: relative;
}

/* Inited */
.panels-layout.process-detail-layout:not(.inited) .panels-layout-actions {
  opacity: 0;
  pointer-events: none;
}
/* Thread comments panel */
.panels-layout .thread-comments-panel:not(.ready) .thread-comments-panel-toolbar-wrapper {
  /* XXX: Can be disabed temporarily */
  display: none;
}
.panels-layout .thread-comments-panel {
  display: flex;
  flex-direction: column;
  overflow: hidden;
}
.panels-layout .thread-comments-panel .panels-layout-content {
  flex: 1;
  overflow: auto;
}
.panels-layout .thread-comments-panel-toolbar-wrapper {
  padding: 10px 20px;
  background-color: #f8f8f8;
  border-bottom: 1px solid #eee;
}
.panels-layout .thread-comments-toolbar > .group,
.panels-layout .thread-comments-toolbar {
  display: flex;
  gap: 5px;
  align-items: center;
}
.panels-layout .thread-comments-toolbar > .group > *,
.panels-layout .thread-comments-toolbar > * {
  margin: 0;
  display: block;
  transition: all var(--common-animation-time);
}
.panels-layout .thread-comments-toolbar > .group {
  padding: 0 5px;
}
.panels-layout .thread-comments-toolbar label {
  opacity: 0.5;
}
.panels-layout .thread-comments-toolbar > a.toolbar-icon {
  background-color: #bbb;
  /* background-color: var(--layout-theme-primary-color); */
  color: #fff;
  border-radius: 4px;
  width: var(--thread-comments-toolbar-items-size);
  height: var(--thread-comments-toolbar-items-size);
  display: flex;
  align-items: center;
  justify-content: center;
  opacity: 0.25;
}
.panels-layout .thread-comments-toolbar > a.toolbar-icon:not(.disabled) {
  cursor: pointer;
  opacity: 0.5;
}
.panels-layout .thread-comments-toolbar:hover > a.toolbar-icon:not(.disabled) {
  background-color: var(--layout-theme-primary-color);
}
.panels-layout .thread-comments-toolbar > a.toolbar-icon:not(.disabled):active,
.panels-layout .thread-comments-toolbar > a.toolbar-icon:not(.disabled):hover {
  opacity: 1;
}
.panels-layout .thread-comments-toolbar > a.toolbar-icon:not(.disabled):active {
  color: var(--layout-theme-primary-color);
  box-shadow: 0 0 4px var(--layout-theme-primary-color);
  background-color: #fff;
}
.panels-layout .thread-comments-panel .thread-comments {
  width: 100%;
}

/* Info section */
.process-detail .info-tableau {
  position: relative;
  transition: all var(--common-animation-time);
  min-height: 0;
}
.process-detail.loading .info-tableau {
  min-height: 50px;
}

/* Error */
.process-detail .info-tableau .error {
  border-radius: 4px;
  background-color: var(--common-error-color);
  color: #fff;
  margin: 20px auto;
  padding: 10px 20px;
}
.process-detail:not(.has-error) .info-tableau .error {
  display: none;
}

/* Loader splash */
.process-detail .info-tableau .loader-splash {
  padding: 20px;
}
.process-detail:not(.loading) .info-tableau .loader-splash {
  /* Inherited from `.loader-splash.hidden`, see `bw_matchbox/assets/css/common.css` */
  opacity: 0;
  pointer-events: none;
}

/* Disabled buttons */
.process-detail button.disabled,
#mark-waitlist-dialog-modal button.disabled {
  opacity: 0.5;
  pointer-events: none;
  cursor: default;
  user-select: none;
}

/* Table cell's widths */
.process-detail .fixed-table .cell-amount {
  width: var(--common-table-cell-amount-width);
}
.process-detail .fixed-table .cell-name {
  width: var(--common-table-cell-name-width);
}
.process-detail .fixed-table .cell-location {
  width: var(--common-table-cell-location-width);
}
.process-detail .fixed-table .cell-unit {
  width: var(--common-table-cell-unit-width);
}
.process-detail .fixed-table .cell-matched {
  width: var(--common-table-cell-matched-width);
}
