/* stylelint-disable
    declaration-block-single-line-max-declarations,
    declaration-colon-space-after,
    selector-class-pattern,
*/

/* Root node */
.processes-list {
  margin-bottom: 20px;
}

/* Empty state: hode some elements... */
.processes-list.empty .processes-list-table {
  display: none;
}
.processes-list.loading .processes-list-table-empty,
.processes-list:not(.empty) .processes-list-table-empty {
  display: none;
}
.processes-list .processes-list-table-empty {
  padding: 10px;
  margin: 10px auto;
  text-align: center;
  opacity: 0.5;
}

/* Loading state: disable some elements... */
.processes-list > * {
  transition: all var(--common-animation-time);
}
.processes-list.loading .page-search,
.processes-list.loading .page-filters,
.processes-list.loading .bottom-actions-links {
  opacity: 0.5;
  pointer-events: none;
}
.processes-list.loading .bottom-actions-links {
  opacity: 0;
}

/* Top radio groups... */
.processes-list .page-filters {
  line-height: 1;
}
.processes-list .page-filters > span,
.processes-list .page-filters > span > span {
  display: inline-block;
  margin-right: 10px;
}
.processes-list .page-filters .radio-group-item {
  white-space: nowrap;
}
.processes-list .page-filters .radio-group-item.disabled {
  opacity: 0.5;
  pointer-events: none;
}
.processes-list .page-filters .radio-group-item > label {
  font-weight: normal;
}
.processes-list.has-search .radio-group.order-by {
  pointer-events: none;
  opacity: 0.5;
}

/* Table */
.processes-list .processes-list-table {
  margin: 20px 0;
}
.processes-list .processes-list-table tr td:last-child div > .button {
  display: block;
  width: 100%;
  margin-bottom: 0;
}
/* Display 'Matched' or 'Reference product' title for 'matched' column respectively by the `userDb` propery. */
.processes-list.userDb-source .processes-list-table .cell-matched-title-other,
.processes-list:not(.userDb-source) .processes-list-table .cell-matched-title-source {
  display: none;
}

/* Table cell's width */
.processes-list .processes-list-table .cell-amount {
  width: var(--common-table-cell-amount-width);
}
.processes-list .processes-list-table .cell-name {
  width: var(--common-table-cell-name-width);
}
.processes-list .processes-list-table .cell-location {
  width: var(--common-table-cell-location-width);
}
.processes-list .processes-list-table .cell-unit {
  width: var(--common-table-cell-unit-width);
}
.processes-list .processes-list-table .cell-matched {
  width: var(--common-table-cell-matched-width);
}

/* Filter controls section */
.processes-list .page-filters label {
  display: inline-block;
  margin-left: 4px;
  margin-right: 8px;
}
.processes-list .page-filters label:first-child {
  margin-left: 0;
}

/* Loader splash */
.processes-list .loader-splash {
  padding: 20px;
}
.processes-list:not(.loading) .loader-splash {
  /* Inherited from `.loader-splash.hidden`, see `bw_matchbox/assets/css/common.css` */
  opacity: 0;
  pointer-events: none;
}

/* Error */
.processes-list .error {
  border-radius: 4px;
  background-color: var(--common-error-color);
  color: #fff;
  margin: 20px auto;
  padding: 10px 20px;
}
.processes-list:not(.error) .error {
  display: none;
}

/* Bottom actions */
.processes-list .bottom-actions {
  position: relative;
  margin: 10px 0;
  min-height: 40px;
  text-align: center;
  display: flex;
  justify-content: center;
  align-items: center;
  align-content: center;
}
/* Action links... */
.processes-list .bottom-actions-links {
  transition: all var(--common-animation-time);
}
.processes-list .bottom-actions-links > * {
  display: none;
  margin: 0 10px;
}
.processes-list .bottom-actions-links > a {
  color: var(--layout-theme-primary-color-dark1);
  cursor: pointer;
}
.processes-list .bottom-actions-links a + a::before {
  /* display: inline-block; */
}
.processes-list .bottom-actions-links a:hover {
  text-decoration: underline;
}
/* Show 'Reload' (after error) button if error... */
.processes-list.error .bottom-actions-links a#action-clear-and-reload,
/* Show 'Load more' button for all 'order by' values except for 'random'... */
.processes-list.has-more-data.has-search:not(.error) .bottom-actions-links a#action-load-more,
.processes-list.has-more-data:not(.order-random, .error) .bottom-actions-links a#action-load-more,
.processes-list:not(.has-more-data, .empty, .error) .bottom-actions-links span#show-total-records-number,
/* Show 'Refresh' button for 'random' 'order by' value... */
.processes-list.order-random:not(.error, .has-search) .bottom-actions-links a#action-reload-random {
  display: inline-block;
}
